#ifndef _MODELINPUT_H
#define _MODELINPUT_H

#include <map>
#include <string>

class ModelInput
{
  // the parameters are intentionally declared outside this class in the L-system,
  // so that if the user doesn't want to read parameters from a file, the values
  // can still be changed the 'old fashioned way', manually in the L-system file.
  public:
        ModelInput (void)
        {
          // Default values of parameters are set in the L-system file
          // here, the "names" of variables are mapped to the location in memory.

          param_names["TRANSPORT_RS"] = &TRANSPORT_RS;
          param_names["KM_LEAF_PRI"] = &KM_LEAF_PRI;

          param_names["KM_INDE_PRI"] = &KM_INDE_PRI;


          param_names["KM_INDE_SEC"] = &KM_INDE_SEC;
          param_names["KM_RESERVES"] = &KM_RESERVES;

          param_names["KM_FRUIT_PRI"] = &KM_FRUIT_PRI;


          param_names["KM_LEAF_SRC"] = &KM_LEAF_SRC;
          param_names["KM_INDE_SRC"] = &KM_INDE_SRC;
          param_names["NUM_CANES_DEFOLIATE"] = &NUM_CANES_DEFOLIATE;
	}

        ~ModelInput (void)
	{
	}

	void readFromFile (char *filename)
        // NOTE: parameter values can be read in from multiple files
        // e.g., two files with half of the parameters each
	{
          FILE *input_file;
          char str[64];
          float value;

          if ((input_file = fopen(filename, "r")) == NULL)
          {
            Printf ("Error: cannot open input file: %s\n", filename);
            return;
          }

          // loop through all entries in the data file
          while (fscanf(input_file, "%s %g", str, &value) != EOF)
          {
            // check if this parameter was declared (i.e. it exists)
            if (param_names[str] == NULL)
              Printf ("Error: from %s file, %s not declared in L-system or in ModelInput\n", filename, str);
            else
              *(param_names[str]) = value;
          }

          fclose(input_file);
          return;
	}

  private:
        std::map<std::string,float*> param_names;
};

#endif
